/**
 * This class holds all information for a Foreign Key object, including heuristic results for the FK.
 */
package de.huberlin.informatik.wbi.aladin.intraschema;

import java.io.Serializable;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Connection;
import java.util.HashMap;
import java.util.Map;

import de.huberlin.informatik.wbi.aladin.util.HeuristicResult;

/**
 * @author Oliver Albrecht (oliver.albrecht@informatik.hu-berlin.de)
 *
 */
public class ForeignKey extends InclusionDependency implements Comparable<ForeignKey>, Serializable{

	private static final long serialVersionUID = -3194983570453611178L;

	/**
	 * Holds all HeuristicResults for this ForeignKey
	 */
	private HashMap<String, HeuristicResult> heuristicResult = new HashMap<String, HeuristicResult>();
	
	/**
	 * Contructor for creating ForeignKex from ResultSet
	 * @param result The Resultset from one Line of Spideroutput
	 * @throws SQLException
	 */
	public ForeignKey(ResultSet result) throws SQLException
	{
		super(result.getString("SCHEMA_NAME"), result.getString("DEPENDENT_TABLE_NAME"),
				result.getString("DEPENDENT_COLUMN_NAME"), result.getString("REFERENCED_TABLE_NAME"),
				result.getString("REFERENCED_COLUMN_NAME"));

	}
	
	/**
	 * Writes the Result from a heuristic into the private HashMap
	 * @param heuristic The Name of the heuristc
	 * @param value The Value of the heuristiv
	 */
	public void writeHeuristicResult(String heuristic, HeuristicResult value)
	{
		this.heuristicResult.put(heuristic, value);
	}
	
	/**
	 * Returns the heuristic result for one Special Heuristic. 
	 * @param heuristicName The Name of the heuristic
	 * @return The Result of the Heuristic
	 */
	public HeuristicResult getHeuristicResult(String heuristicName)
	{
		if(!this.heuristicResult.containsKey(heuristicName))
		{
			return new HeuristicResult();
		}
		else
		{
			return this.heuristicResult.get(heuristicName);
		}
	}
	
	/**
	 * Resturns the Hashmap with the Result for all heuristics
	 * @return Haspmap with elements <heuriticName> -> <result>
	 */
	public HashMap<String, HeuristicResult> getHeuriticsResult()
	{
		return this.heuristicResult;
	}
	
	/**
	 * Returns the number of positiv results that this key is an foreign key.
	 * @return
	 */
	public int getNumberOfPostivResults()
	{
		int count = 0;
		for(Map.Entry<String, HeuristicResult> entry : this.heuristicResult.entrySet())
		{
			if(((HeuristicResult)entry.getValue()).isForeignKey())
			{
				count++;
			}
		}
		return count;
	}
	
	/**
	 * Returns the number of negativ results that this key is an foreign key.
	 * @return
	 */
	public int getNumberOfNegativResults()
	{
		int count = 0;
		for(Map.Entry<String, HeuristicResult> entry : this.heuristicResult.entrySet())
		{
			if(((HeuristicResult)entry.getValue()).isNoForeignKey())
			{
				count++;
			}
		}
		return count;
	}
	
	/**
	 * Returns the Probability that this is an Foreign Key.
	 * The result is counted from all Heuristic results.
	 * @return
	 */
	public int getProbability()
	{
		int result = 0;
		for( Map.Entry<String, HeuristicResult> entry : this.heuristicResult.entrySet())
		{
			if(((HeuristicResult)entry.getValue()).isForeignKey())
			{
				result += ((HeuristicResult)entry.getValue()).getEmphasis();
			}
		}
		
		return result;
	}
	
	/**
	 * Returns the maximum Probabilityvalue that was possible to get.
	 * @return
	 */
	public int getMaxProbability()
	{
		int result = 0;
		for( Map.Entry<String, HeuristicResult> entry : this.heuristicResult.entrySet())
		{
			if(((HeuristicResult)entry.getValue()).hasClearResult())
			{
				result += ((HeuristicResult)entry.getValue()).getEmphasis();
			}
		}
		
		return result;
	}
	
	/**
	 * Calculatest the Percentage of how high is the probability that the IND is an FK
	 * @return
	 */
	public float getProbabilityPercentage()
	{
		return ((float)this.getProbability() / (float)this.getMaxProbability()) * 100;
	}
	
	/**
	 * Writes the Result of SearchForeignKey into the database
	 * @param conn The Connection on which the update should be performed
	 * @param schemaName The Schemaname on which the Update will be performed
	 * @param tableName The tablename on which the Update will be performed
	 */
	public void writeResultToDatabase(Connection conn, String schemaName, String tableName)
	{
		try
		{
			StringBuffer update = new StringBuffer();
			update.append("Update ").append(schemaName.toUpperCase()).append(".").append(tableName)
				.append(" SET POSITIVE_HEURISTIC_RESULTS = ").append(this.getNumberOfPostivResults())
				.append(", NUMBER_OF_HEURISTICS = ")
				.append(this.getNumberOfNegativResults() + this.getNumberOfPostivResults())
				.append(", PROBABILITY = ").append(this.getProbability())
				.append(", MAX_PROBABILITY = ").append(this.getMaxProbability())
				.append(" WHERE SCHEMA_NAME = '").append(this.getSchemaName())
				.append("' AND DEPENDENT_TABLE_NAME = '").append(this.getDependentTable())
				.append("' AND DEPENDENT_COLUMN_NAME = '").append(this.getDependentColumn())
				.append("' AND REFERENCED_TABLE_NAME = '").append(this.getReferencedTable())
				.append("' AND REFERENCED_COLUMN_NAME = '").append(this.getReferencedColumn())
				.append("'");
			Statement table = conn.createStatement();
			table.executeUpdate(update.toString());
			table.close();
		}
		catch(SQLException e)
		{
			e.printStackTrace();
		}
	}
	
	/**
	 * Compares another Foreign Key with the actual one. First compared value
	 * is the Percentage of probability, second compared value is the absolut
	 * number of reached points
	 * 
	 * @param ForeignKey for comparism
	 * @return 
	 */
	public int compareTo(ForeignKey key)
	{
		int returnValue = new Float(this.getProbabilityPercentage())
	 	.compareTo(new Float(key.getProbabilityPercentage()));
		if(returnValue == 0)
		{
			// compare on the maximum Number of reached Points
			returnValue = new Float(this.getMaxProbability())
		 	.compareTo(new Float(key.getMaxProbability()));
		}
		
		return -returnValue; 
	}
}
