/**
 * This class got all functionality for finding Foreign Keys in Inclusiondependencies.
 */
package de.huberlin.informatik.wbi.aladin.intraschema;

import java.sql.Connection;
import java.sql.Statement;
import java.sql.SQLException;
import java.util.Properties;
import java.util.Vector;
import java.util.Map;
import java.util.Collections;
import java.sql.ResultSet;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.ProgressBar;

import de.huberlin.informatik.wbi.aladin.util.HeuristicResult;
import de.huberlin.informatik.wbi.aladin.util.IlleagalHeuristicName;
import de.huberlin.informatik.wbi.aladin.intraschema.fkfinderGui.FkfinderGui;
import de.huberlin.informatik.wbi.aladin.intraschema.fkfinderGui.FkfinderGuiWorking;
import de.huberlin.informatik.wbi.aladin.intraschema.heuristics.*;

/**
 * @author Oliver Albrecht (oliver.albrecht@informatik.hu-berlin.de)
 *
 */
public class ForeignKeyFinder {
	
	/**
	 * Logger for logging.
	 */
	static Logger logger = Logger.getLogger(ForeignKeyFinder.class);
	
	/**
	 * Propertiesobject, holding the Configuration for the run
	 */
	private Properties config;
	
	/**
	 * Holds the onnection to the Database.
	 */
	private Connection conn;
	
	/**
	 * Holds the Schemaname where the spideroutput is saved.
	 */
	private String schemaName;
	
	/**
	 * Holds the tableName where the Spideroutput is saved.
	 */
	private String tableName;
	
	/**
	 * Holds the schema that should be checked.
	 */
	private String checkSchema;
	
	/**
	 * Holds all Foreign Keys in a List
	 */
	private Vector<ForeignKey> foreignKeys = new Vector<ForeignKey>();
	
	/**
	 * Holds the time it took for running all heuristics
	 */
	private long timeForFinding;
	
	/**
	 * Holds the GUI Object for status updates.
	 */
	private FkfinderGuiWorking Gui = null;
	
	/**
	 * Holds a flag if the GUI should be updated
	 */
	private Boolean updateGui = false;
	
	/**
	 * Creates a new Object of ForeignKeyFinder
	 * @param conn Connection to the Database
	 * @param schemaName Name of the Schema where the Spideroutput is saved
	 * @param tableName Name of the Table where the spideroutput is saved
	 * @param checkSchema Name of the Schema that will be checked
	 */
	public ForeignKeyFinder(Properties config, Connection conn, String schemaName, 
			String tableName, String checkSchema)
	{
		this.config = config;
		this.conn = conn;
		this.schemaName = schemaName;
		this.tableName = tableName;
		this.checkSchema = checkSchema;
		
		logger.setLevel(Level.toLevel(this.config.getProperty("Logger.Level").trim()));
	}
	
	/**
	 * Saves an GUI object for updates.
	 * If this method is called, all updates of the status are send to GUI.
	 * @param Gui
	 */
	public void setUpdateGui(FkfinderGuiWorking Gui)
	{
		this.Gui = Gui;
		this.updateGui = true;
	}
	
	/**
	 * Starts the Search for foreign Keys. All configured Heuristics are started on the
	 * found inclusion dependencies.
	 */
	public void find()
	{
		long startTime = System.currentTimeMillis();
		logger.debug("Starting search for Foreign keys");
		
		// Read all Depencies from Spideroutput and put them into collection
		try
		{
			StringBuffer inds = new StringBuffer(200);
			inds.append("Select * FROM ").append(this.schemaName).append(".")
				.append(this.tableName).append(" WHERE SCHEMA_NAME = '")
				.append(this.checkSchema.toUpperCase()).append("'");
			logger.debug(inds.toString());
			Statement table = conn.createStatement();
			ResultSet rs = table.executeQuery(inds.toString());
			while(rs.next())
			{
				try
				{
					ForeignKey key = new ForeignKey(rs);
					this.foreignKeys.add(key);
				}
				catch(SQLException e)
				{
					logger.fatal("Could not create Foreign Key Object because of Database error:");
					e.printStackTrace();
					System.exit(5);
				}
			}
			rs.close();
			table.close();
		}
		catch(SQLException e)
		{
			logger.fatal("Could not read Inclusiondependencies");
			System.exit(5);
		}
		
		// start all Heuristics ForeignKeyFinder.HeuristicsEmphasis
		String[] emphasis = this.config.getProperty(
				"ForeignKeyFinder.HeuristicsEmphasis").split(",");
		String[] heuristics = this.config.getProperty(
				"ForeignKeyFinder.HeuristicsToUse").split(",");
		if(emphasis.length != heuristics.length)
		{
			logger.fatal("Wrong Configuration: Number of " +
					"Heuristics differs to number of emphasises");
			System.exit(5);
		}
		
		for(int i = 0; i < heuristics.length; i++)
		{
			try
			{
				if(this.updateGui)
				{
					this.Gui.updateHeuristics(i, heuristics.length, "Heursitic: " + heuristics[i]);
				}
				ForeignKeyHeuristic heuristicObj = ForeignKeyHeuristic.getHeuristicObject
					(heuristics[i], this.conn, this.config, this.foreignKeys);
				long timeTemp = System.currentTimeMillis();
				heuristicObj.setEmphasis(new Integer(emphasis[i]));
				if(this.updateGui)
				{
					heuristicObj.setUpdateGui(this.Gui);
				}
				heuristicObj.findForeignKeys();
				if(this.updateGui)
				{
					this.Gui.updateHeuristics(i + 1, heuristics.length, "Heursitic: " + heuristics[i]);
				}
				logger.info("finished (" + heuristicObj.getClass().getSimpleName() + ") in " + 
						((System.currentTimeMillis() - timeTemp) / 1000) + "s");
				
			}
			catch (IlleagalHeuristicName e)
			{
				logger.error(e.getMessage());
			}
		}
		if(this.updateGui)
		{
			this.Gui.updateHeuristics(heuristics.length, heuristics.length, "Evaluating result...");
		}
			
		this.timeForFinding = System.currentTimeMillis() - startTime;
	}
	
	/**
	 * Saves the Result of every key into the Database.
	 *
	 */
	public void saveResult()
	{
		boolean writeToDB = 
			new Boolean(this.config.getProperty("ForeignKeyFinder.writeResultIntoDatabase")); 
		if(writeToDB)
		{
			logger.debug("Writing Results to Database");
			for(ForeignKey key : this.foreignKeys)
			{
				key.writeResultToDatabase(this.conn, this.schemaName, this.tableName);
			}
		}
	}
	
	/**
	 * prints the Result for all Dependencies in the schema to stdOut
	 *
	 */
	public void printResult()
	{
		boolean writeToDB = 
			new Boolean(this.config.getProperty("ForeignKeyFinder.printResultsToShell"));
		if(writeToDB)
		{
			Collections.sort(this.foreignKeys);
			logger.debug("Printing Results");
			System.out.println("Results for Schema '" + this.checkSchema + "'");
			System.out.println("Time: " + (this.getTime() / 1000) + "s");

			for (ForeignKey key : this.foreignKeys)
			{
				System.out.println(key.toString());
				System.out.println("Result: " + key.getProbability() + "/" 
						+ key.getMaxProbability() + " ->" + key.getProbabilityPercentage() + "%");
				System.out.println("Heuristics: " + key.getNumberOfPostivResults() + "/" 
						+ (key.getNumberOfNegativResults() + key.getNumberOfPostivResults()));
				for(Map.Entry entry : key.getHeuriticsResult().entrySet())
				{
					System.out.println("  " + (String)entry.getKey() 
							+ " --> " + ((HeuristicResult)entry.getValue()).toString());
				}
				System.out.println();
			}
		}
	}
	
	/**
	 * Returns the time it took for running all heuristics.
	 * @return
	 */
	public long getTime()
	{
		return this.timeForFinding;
	}
	
	/**
	 * Returns the Result of the finding process.
	 * @return The result is returned as Vector of Foreign Keys.
	 */
	public Vector<ForeignKey> getResult()
	{
		return this.foreignKeys;
	}

}
