/**
 * This class is needed for starting the search for Foreign Keys out of
 * inclusion dependencies. It can be used directly from the shell or
 * from a GUI.
 */
package de.huberlin.informatik.wbi.aladin.intraschema;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.Statement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.io.FileInputStream;
import java.util.Properties;
import java.util.Vector;
import java.io.IOException;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.apache.log4j.BasicConfigurator;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.ProgressBar;

import de.huberlin.informatik.wbi.aladin.intraschema.ForeignKeyFinder;
import de.huberlin.informatik.wbi.aladin.intraschema.fkfinderGui.FkfinderGui;
import de.huberlin.informatik.wbi.aladin.intraschema.fkfinderGui.FkfinderGuiWorking;
import de.huberlin.informatik.wbi.aladin.util.ForeignKeyFinderRunException;

/**
 * @author Oliver Albrecht (oliver.albrecht@informatik.hu-berlin.de)
 *
 */
public class ForeignKeyFinderExec extends Thread{

	/**
	 * Logger for logging
	 */
	static Logger logger = Logger.getLogger(ForeignKeyFinderExec.class.getName());
	
	/**
	 * The standardconfigurationfile if no file is specified
	 */
	private String configurationFile = "cfg/ForeignKeyFinder.properties"; 
	
	/**
	 * Holds the configuration for the finder
	 */
	private Properties config;
	
	/**
	 * holds the schema where Spider output is hold
	 */
	private String schemaName;
	
	/**
	 * holds the table name where Spider output is saved to
	 */
	private String tableName;
	
	/**
	 * holds the Schema that should be checked
	 */
	private String checkSchema;
	
	/**
	 * Holds the GUI Object for status updates.
	 */
	private FkfinderGuiWorking Gui = null;
	
	/**
	 * Holds a flag if the GUI should be updated
	 */
	private Boolean updateGui = false;
	
	/**
	 * Holds the finder Object.
	 */
	private ForeignKeyFinder find = null;
	
	/**
	 * Mainly used for start vie shell, when config object has to be created inside the finder.
	 */
	public ForeignKeyFinderExec(){BasicConfigurator.configure();}
	
	/**
	 * Mainly used for start via GUI if a configobject was already created.
	 * @param config
	 */
	public ForeignKeyFinderExec(Properties config)
	{
		BasicConfigurator.resetConfiguration();
		BasicConfigurator.configure();
		this.config = config;
		this.setSchemaName();
		this.setTableName();
		this.setCheckSchema();
		
	}
	
	/**
	 * Starts the Object as Thread
	 */
	public void run()
	{
		try
		{
			startFkFinder();
		}
		catch(ForeignKeyFinderRunException e)
		{
			if(this.updateGui == false)
			{
				System.exit(5);
			}
			this.Gui.errorOccured(e.getMessage());
		}
	}
	
	/**
	 * For Start via shell
	 * @param args 
	 * 	          <ul>
	 *            	<li>[&lt; ConfigurationFile  &gt;]</li>
	 *            </ul>
	 */
	public static void main(String[] args) {
		
		ForeignKeyFinderExec finder = new ForeignKeyFinderExec();
		
		// Check Arguments
		if(finder.readConfiguration(args) == false)
		{
			finder.printUsageAndExit();
		}
		try
		{
			finder.startFkFinder();
		}
		catch(ForeignKeyFinderRunException e)
		{
			System.exit(5);
		}
		
	}
	
	/**
	 * Saves an GUI object for updates.
	 * If this method is called, all updates of the status are send to GUI.
	 * @param Gui
	 */
	public void setUpdateGui(FkfinderGuiWorking Gui)
	{
		this.Gui = Gui;
		this.updateGui = true;
	}
	
	/**
	 * Starts the FKFinder. For Start via shell or GUI
	 */
	public void startFkFinder() throws ForeignKeyFinderRunException
	{
		// try DB connection
		logger.debug("starting connection to databse ...");
		try{
			
			DriverManager.registerDriver(new oracle.jdbc.OracleDriver());
			Connection conn = DriverManager
				.getConnection(this.config.getProperty("Database.connectString"), 
						this.config.getProperty("Database.user"), 
						this.config.getProperty("Database.password"));
			// Settings for String comparison
			Statement stmt = conn.createStatement();
            stmt.execute("ALTER SESSION SET NLS_COMP=ansi");
            stmt.execute("ALTER SESSION SET NLS_SORT=unicode_binary");
            stmt.close(); 
			
			// Check if table is available and has content for requested Schema
			StringBuffer checkTable = new StringBuffer(200);
			checkTable.append("Select count(*) FROM ").append(this.getSchemaName()).append(".")
				.append(this.getTableName()).append(" WHERE SCHEMA_NAME = '")
				.append(this.getCheckSchema()).append("'");
			logger.debug(checkTable.toString());
			Statement table = conn.createStatement();
			ResultSet result = table.executeQuery(checkTable.toString());
			result.next();
			long NumberOfDependencies = result.getLong(1);
			if(NumberOfDependencies == 0)
			{
				logger.fatal("No Information about Schema " + this.getCheckSchema() + " in Database");
				if(this.updateGui)
				{
					throw new ForeignKeyFinderRunException("No Information about Schema " 
							+ this.getCheckSchema() + " in Database");
				}
				else
				{
					this.printUsageAndExit();
				}
			}
			else
			{
				logger.debug("Found " + NumberOfDependencies + " Dependencies to check.");
			}
			result.close();
			table.close();
			
			// start Finder
			logger.debug("Database okay, starting ForeignKeyFinder ...");
			
			this.find = new ForeignKeyFinder(this.config, conn, this.getSchemaName(), 
					this.getTableName(), this.getCheckSchema());
			if(this.updateGui)
			{
				this.find.setUpdateGui(this.Gui);
			}
			this.find.find();
			
			// print and write the result
			this.find.saveResult();
			this.find.printResult();
			
			// Every Heuristic should be finished now, closing db connection and exit.
			conn.close();
			logger.debug("Closing Database connection, ForeignKeyFinderFinished.");
			if(this.updateGui == false)
			{
				System.exit(0);
			}
			else
			{
				this.Gui.finderFinished();
			}
			
		}catch(SQLException e)
		{
			logger.fatal("Could not connect to database");
			e.printStackTrace();
			if(this.updateGui == false)
			{
				System.exit(5);
			}
			throw new ForeignKeyFinderRunException(e.getMessage());
		}
		catch(Exception e)
		{
			if(this.updateGui == false)
			{
				System.exit(5);
			}
			throw new ForeignKeyFinderRunException(e.getMessage());
		}
	}
	
	/**
	 * Checks if the configurationfile is readable. If not it returns false.
	 * 
	 * @param args The arguments that where given for the start of the tool
	 * @return
	 */
	private boolean readConfiguration(String[] args)
	{
		this.config = new Properties();
		if(args.length > 0)
		{
			this.configurationFile = args[0];
		}
				
		// try to read ConfigurationFile
		try
		{
			this.config.load(new FileInputStream(this.configurationFile));
		}
		catch(IOException e)
		{
			logger.fatal("Could not read Configurationfile:" + this.configurationFile);
			return false;
		}
		logger.setLevel(Level.toLevel(this.config.getProperty("Logger.Level").trim()));
		logger.debug("Configurationfile was read succesfully: " + this.configurationFile);
		if(!this.setSchemaName())
		{
			return false;
		}
		if(!this.setTableName())
		{
			return false;
		}
		if(!this.setCheckSchema())
		{
			return false;
		}
		logger.debug("Configurationfile was set succesfully");
		return true;
	}
	
	/**
	 * Sets the Schemane where Spideroutput is saved.
	 * @return Returns true if the Schemaname is found, else false
	 */
	private boolean setSchemaName()
	{
		this.schemaName = this.config.getProperty("ForeignKeyFinder.schemaName");
		if(this.schemaName == null)
		{
			logger.fatal("Could not read schemaName from propertiesfile");
			return false;
		}

		return true;
	}
	
	/**
	 * Sets the TableName where the spideroutput is saved
	 * @return return true if the tableName is found, else false
	 */
	private boolean setTableName()
	{
		this.tableName = this.config.getProperty("ForeignKeyFinder.tableName");
		if(this.tableName == null)
		{
			logger.fatal("Could not read tableName from propertiesfile");
			return false;
		}

		return true;
	}
	
	/**
	 * Sets the CheckSchemaName
	 * @return If the list includes the CheckSchemaName it return s true, else false
	 */
	private boolean setCheckSchema()
	{
		this.checkSchema = this.config.getProperty("ForeignKeyFinder.checkSchema");
		if(this.checkSchema == null)
		{
			logger.fatal("Could not read checkSchema from propertiesfile");
			return false;
		}

		return true;
	}
	
	/**
	 * 
	 * @return the Name of the Schema where the Spider output is stored.
	 */
	public String getSchemaName()
	{
		return this.schemaName;
	}
	
	/**
	 * 
	 * @return The Name of the Table where the Spideroutput is stored
	 */
	public String getTableName()
	{
		return this.tableName;
	}
	
	/**
	 * 
	 * @return The Name of The Schema which should be checked
	 */
	public String getCheckSchema()
	{
		return this.checkSchema;
	}
	
	/**
	 * Returns the Vector holding the result of the search.
	 * @return
	 */
	public Vector<ForeignKey> getResult()
	{
		return this.find.getResult();
	}
	
	/**
	 * Returns the config object from the finder.
	 * @return
	 */
	public Properties getConfig()
	{
		return this.config;
	}
	
	/**
	 * Prints out the usage of ForeignKeyFinderExec and exits the VM
	 *
	 */
    private void printUsageAndExit() {
        System.err.println(
                "usage: ForeignKeyFinderExec [<configurationFile>]\n\t");
        System.exit(5);      
    }

}
