/**
 * And Abstract class with all the important methods that must be implemented by a heuristic.
 * All heuristics should implement this class.
 */
package de.huberlin.informatik.wbi.aladin.intraschema.heuristics;

import java.util.Properties;
import java.util.Vector;
import java.sql.Connection;

import org.apache.log4j.Logger;
import org.apache.log4j.Level;

import de.huberlin.informatik.wbi.aladin.intraschema.ForeignKey;
import de.huberlin.informatik.wbi.aladin.intraschema.fkfinderGui.FkfinderGuiWorking;
import de.huberlin.informatik.wbi.aladin.util.HeuristicResult;
import de.huberlin.informatik.wbi.aladin.util.IlleagalHeuristicName;

/**
 *@author Oliver Albrecht (oliver.albrecht@informatik.hu-berlin.de)
 *
 */
public abstract class ForeignKeyHeuristic {
	
	/**
	 * Logger for logging
	 */
	protected Logger logger = Logger.getLogger(ForeignKeyHeuristic.class);
	
	/**
	 * Holds the configuration Object with all Parameter for this heuristic
	 */
	protected Properties config;
	
	/**
	 * Holds the Collection of the Inclusiondependencies that should be tested
	 */
	protected Vector<ForeignKey> foreignKeys;
	
	/**
	 * Holds a connection Object for Databse requests
	 */
	protected Connection conn;
	
	/**
	 * Holds all configuration parameter that are used by that Heuristic
	 */
	protected Vector<String> properties = new Vector<String>();
	
	/**
	 * Holds the counter for the emphasis of this heuristic in the later Overallresult.
	 */
	protected int emphasis = 0;
	
	/**
	 * A GUI object for making an update of the current status.
	 */
	private FkfinderGuiWorking Gui;
	private boolean updateGui = false;
	
	/**
	 * Construct of a Heuristic.
	 * @param config Configuratinobject with all parameter for the Heuristic
	 * @param foreignKeys Vector of the ForeignKeys that should be tested
	 */
	public ForeignKeyHeuristic(Connection conn, Properties config, Vector<ForeignKey> foreignKeys)
	{
		this.conn = conn;
		this.config = config;
		this.foreignKeys = foreignKeys;
		this.logger.setLevel(Level.toLevel(this.config.getProperty("Logger.Level").trim()));
	}
	
	/**
	 * Saves an GUI object for updates.
	 * If this method is called, all updates of the status are send to GUI.
	 * @param Gui
	 */
	public void setUpdateGui(FkfinderGuiWorking Gui)
	{
		this.Gui = Gui;
		this.updateGui = true;
	}
	
	/**
	 * Returns an Object for a special Heuristic. 
	 * @param heuristicName Name of the heuristic
	 * @param conn @see ForeignKeyHeuristic()
	 * @param config @see ForeignKeyHeuristic()
	 * @param foreignKeys @see ForeignKeyHeuristic()
	 * @return @see ForeignKeyHeuristic()
	 * @throws IlleagalHeuristicName Is thrown in case of an unknown name
	 */
	public static ForeignKeyHeuristic getHeuristicObject(String heuristicName, Connection conn, 
			Properties config, Vector<ForeignKey> foreignKeys) throws IlleagalHeuristicName
	{
		if(heuristicName.equalsIgnoreCase("ForeignKeyHeuristicMinNumberDependendValues"))
		{
			return new ForeignKeyHeuristicMinNumberDependendValues(conn, config, foreignKeys);
		}
		else if(heuristicName.equalsIgnoreCase("ForeignKeyHeuristicCovering"))
		{
			return new ForeignKeyHeuristicCovering(conn, config, foreignKeys);
		}
		else if(heuristicName.equalsIgnoreCase("ForeignKeyHeuristicNumberOfReferences"))
		{
			return new ForeignKeyHeuristicNumberOfReferences(conn, config, foreignKeys);
		}
		else if(heuristicName.equalsIgnoreCase("ForeignKeyHeuristicColumnName"))
		{
			return new ForeignKeyHeuristicColumnName(conn, config, foreignKeys);
		}
		else if(heuristicName.equalsIgnoreCase("ForeignKeyHeuristicPropotionalValueLength"))
		{
			return new ForeignKeyHeuristicPropotionalValueLength(conn, config, foreignKeys);
		}
		else if(heuristicName.equalsIgnoreCase("ForeignKeyHeuristicPropotionalValueAmount"))
		{
			return new ForeignKeyHeuristicPropotionalValueAmount(conn, config, foreignKeys);
		}
		else if(heuristicName.equalsIgnoreCase("ForeignKeyHeuristicShortTables"))
		{
			return new ForeignKeyHeuristicShortTables(conn, config, foreignKeys);
		}
		else if(heuristicName.equalsIgnoreCase("ForeignKeyHeuristicSpreadingOverReference"))
		{
			return new ForeignKeyHeuristicSpreadingOverReference(conn, config, foreignKeys);
		}
		else
		{
			throw new IlleagalHeuristicName("Unknown Heuristicname: " + heuristicName);
		}
	}
	
	/**
	 * Calculates for all Inclusiondependencies the probability if it is a foreign key
	 * @return true if calculation was successful, false if there was an error
	 */
	public boolean findForeignKeys()
	{
		this.logger.info("Starting Heuristic " + this.getClass().getSimpleName() + " for " 
				+ this.foreignKeys.size() + " dependencies");
		boolean returnValue = true;
		int i = 1;
		for(ForeignKey key : this.foreignKeys)
		{
			HeuristicResult result = this.calculateProbability(key);
			result.setEmphasis(this.getEmphasis());
			this.writeResult(key, result);
			if(this.updateGui)
			{
				this.Gui.updateInds(i, this.foreignKeys.size());
			}
			i++;
		}
		
		return returnValue;
	}
	
	/**
	 * Calculatest the probability for one Inclusiondependencie.
	 * Must be implementet in each heuristic.
	 * @param key The InclusionDependencies which should be tested
	 * @return The probability if the Dependencie is a Foreign Key. Returns -1 in case
	 * 			of an error.
	 */
	public abstract HeuristicResult calculateProbability(ForeignKey key);
	
	/**
	 * Writes the Result of findForeignKeys into the key 
	 * @param key The Inclusiondependencie for which a value was calculated
	 * @param value The value that was calculatet
	 */
	protected void writeResult(ForeignKey key, HeuristicResult value)
	{
		key.writeHeuristicResult(this.getClass().getSimpleName(), value);
	}
	
	/**
	 * Sets the emphasis for this heuristic. Is important for later overall result.
	 * @param emphasis
	 */
	public final void setEmphasis(int emphasis)
	{
		this.emphasis = emphasis;
	}
	
	/**
	 * Returns the emphasis for this heuristic, for the overall result. 
	 * @return
	 */
	public final int getEmphasis()
	{
		return this.emphasis;
	}
	
	/**
	 * Check if all properties are available that are needed by this Heuristic. 
	 * @return True if all properties are available, else false
	 */
	protected boolean checkProperties()
	{
		for (String propertie : this.properties)
		{
			if(this.config.getProperty(propertie) == null)
			{
				this.logger.fatal("Propertie for Heuristic" + this.getClass().getName() 
						+ " missing: " + propertie);
				return false;
			}
		}
		return true;
	}
}
