/**
 * This Heuristic checks the number of values in the referenced attribute, that are outside
 * the border given by min() and max() of the depend attribut. Only a configured part of values 
 * are allowed to be outside.
 */
package de.huberlin.informatik.wbi.aladin.intraschema.heuristics;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Properties;
import java.util.Vector;

import de.huberlin.informatik.wbi.aladin.intraschema.ForeignKey;
import de.huberlin.informatik.wbi.aladin.util.HeuristicResult;
import de.huberlin.informatik.wbi.aladin.util.HeuristicResultValue;

/**
 * @author Oliver Albrecht (oliver.albrecht@informatik.hu-berlin.de)
 *
 */
public class ForeignKeyHeuristicSpreadingOverReference extends ForeignKeyHeuristic {

	/**
	 * @param conn
	 * @param config
	 * @param foreignKeys
	 */
	public ForeignKeyHeuristicSpreadingOverReference(Connection conn, Properties config,
			Vector<ForeignKey> foreignKeys) {
		super(conn, config, foreignKeys);
		
		this.properties.add("Heuristics.ForeignKeyHeuristicSpreadingOverReference.ValuesOutsideMinMaxPercentage");
		this.properties.add("Heuristics.ForeignKeyHeuristicSpreadingOverReference.CheckColumnInfo");
	}

	/* (non-Javadoc)
	 * @see de.huberlin.informatik.wbi.aladin.intraschema.heuristics.ForeignKeyHeuristic#calculateProbability(de.huberlin.informatik.wbi.aladin.intraschema.ForeignKey)
	 */
	@Override
	public HeuristicResult calculateProbability(ForeignKey key) {
		
		boolean useColumnInfo = new Boolean(this.config.getProperty(
			"Heuristics.ForeignKeyHeuristicSpreadingOverReference.CheckColumnInfo"));
		
		float valuesOutside = new Float(this.config.getProperty(
			"Heuristics.ForeignKeyHeuristicSpreadingOverReference.ValuesOutsideMinMaxPercentage")).floatValue();
		
		try
		{
			StringBuffer request = new StringBuffer();
			
			// Min and Max value in Dependent Attribut
			request.append("SELECT min(\"").append(key.getDependentColumn()).append("\")")
				.append(", max(\"").append(key.getDependentColumn()).append("\")")
				.append(" FROM ").append(key.getSchemaName()).append(".\"")
				.append(key.getDependentTable()).append("\"");
			this.logger.debug(request.toString());
			Statement stmt = conn.createStatement();
			ResultSet result = stmt.executeQuery(request.toString());
			result.next();
			
			String minValue = result.getString(1);
			String maxValue = result.getString(2);
			
			stmt.close();
			result.close();
			
			// Read number of values in table
			request = new StringBuffer();
			if(useColumnInfo)
			{
				request.append("SELECT NUM_DISTINCT_COLUMN FROM ")
					.append(this.config.getProperty("Database.columnInfoTable"))
					.append(" WHERE SCHEMA_NAME = '").append(key.getSchemaName())
					.append("' AND TABLE_NAME = '").append(key.getReferencedTable())
					.append("' AND COLUMN_NAME = '").append(key.getReferencedColumn())
					.append("'");
			}
			else
			{
				request.append("SELECT count(DISTINCT \"").append(key.getReferencedColumn())
					.append("\") FROM ").append(key.getSchemaName()).append(".\"")
					.append(key.getReferencedTable()).append("\"");
			}
			this.logger.debug(request.toString());
			stmt = conn.createStatement();
			result = stmt.executeQuery(request.toString());
			result.next();
			float numberOfValues = result.getFloat(1);		
			
			stmt.close();
			result.close();
			
			// Check, how much values are in the referenzed Column outside min max
			request = new StringBuffer();
			request.append("SELECT count(\"").append(key.getReferencedColumn())
				.append("\") FROM ").append(key.getSchemaName()).append(".\"")
				.append(key.getReferencedTable()).append("\"")
				.append(" WHERE \"").append(key.getReferencedColumn())
				.append("\" < '").append(minValue.replace("'", "''")).append("' OR \"")
				.append(key.getReferencedColumn()).append("\" > '").append(maxValue.replace("'", "''"))
				.append("'");
			this.logger.debug(request.toString());
			stmt = conn.createStatement();
			result = stmt.executeQuery(request.toString());
			result.next();
			
			float numberOfValuesOuter = result.getFloat(1);
			
			HeuristicResultValue resultValue = HeuristicResultValue.FALSE;
			if(numberOfValuesOuter < ((numberOfValues / 100) * valuesOutside))
			{
				resultValue = HeuristicResultValue.TRUE;
			}
			
			// Verbindung schliessen
			result.close();
			stmt.close();
			
			this.logger.debug("Set Probability for Column " + key.getDependentColumn() + ": " + resultValue);
			return new HeuristicResult(resultValue, "Number of values ref: " 
					+ numberOfValues + " Number of values outer: " + numberOfValuesOuter
					+ "(" + (numberOfValuesOuter / numberOfValues) * 100 + "%)"
					+ " max allowed: " + ((numberOfValues / 100) * valuesOutside) );
		}
		catch(SQLException e)
		{
			this.logger.fatal("Could not read Dependencie: " + key.toString());
			e.printStackTrace();
		}
		
		return new HeuristicResult();
	}

}
